@@ -2,6 +2,8 @@
 
 一个 Python 迭代器的包装器，使其具有与Rust中的其他方法类似的风格，以提高迭代器操作的一致性和代码的可读性。
 """
+import builtins
+import collections
 import operator
 import sys
 from itertools import (
@@ -44,17 +46,19 @@ from typing import (
 
 from typing_extensions import (
     Protocol,
+    Self,
     SupportsIndex,
     runtime_checkable,
 )
 
 __all__ = ["ArkoWrapper"]
 
-Wrapper = TypeVar("Wrapper", bound="ArkoWrapper")
 T = TypeVar("T")
 E = TypeVar("E")
 R = TypeVar("R")
+Wrapper = TypeVar("Wrapper", bound="ArkoWrapper")
 default_max = sys.maxsize
+NOT_SET = object()
 
 
 @runtime_checkable
@@ -67,12 +71,7 @@ class Searchable(Protocol[T]):
 
 
 class ArkoWrapper(Generic[T]):
-    """一个 Python 迭代器的包装器
-
-    Attributes:
-        iterable: 需要被 Wrap 的 Object。 可以是一个迭代器或者其它任何Object。
-        max_operate_times: Wrapper 操作次数的上限。用于限制无限的迭代器。
-    """
+    """一个 Python 迭代器的包装器"""
     __root__: Iterable[T]
     _max: int
 
@@ -83,6 +82,12 @@ class ArkoWrapper(Generic[T]):
             self, iterable: Optional[Union[Iterable[T], T]] = None, *,
             max_operate_times: Optional[int] = default_max
     ) -> NoReturn:
+        """初始化方法
+
+        Args:
+            iterable: 需要被 Wrap 的 Object。 可以是一个迭代器或者其它任何Object。
+            max_operate_times: Wrapper 操作次数的上限。用于限制无限的迭代器。
+        """
         if isinstance(iterable, Iterable):
             self.__root__ = iterable
         elif iterable is None:
@@ -104,10 +109,24 @@ class ArkoWrapper(Generic[T]):
 
     def _tee(self) -> Iterable[T]:
         """将已有迭代器分裂一次"""
-        result, self.__root__ = tee(self.__root__)
+        iter_values = iter(self.__root__)
+        deques = (collections.deque(), collections.deque())
+
+        def generator(deque) -> Iterator[T]:
+            while True:
+                if not deque:
+                    try:
+                        new_val = next(iter_values)
+                    except StopIteration:
+                        return
+                    for d in deques:
+                        d.append(new_val)
+                yield deque.popleft()
+
+        result, self.__root__ = generator(deques[0]), generator(deques[1])
         return result
 
-    def _max_gen(self) -> Iterator:
+    def _max_gen(self) -> Iterator[T]:
         """将自己的迭代器现在某个范围内"""
         iter_values = iter(self._tee())
         try:
@@ -116,7 +135,7 @@ class ArkoWrapper(Generic[T]):
         except StopIteration:
             pass
 
-    def __add__(self, other: Union[E, Iterable[E]]) -> Wrapper:
+    def __add__(self, other: Union[E, Iterable[E]]) -> Self:
         """实现加法操作。返回两个实列组成的新的迭代器的ArkoWrapper
 
         Args:
@@ -135,7 +154,7 @@ class ArkoWrapper(Generic[T]):
 
         return self.__class__(generate())
 
-    def __radd__(self, other: Union[Iterable[E], E]) -> Wrapper:
+    def __radd__(self, other: Union[Iterable[E], E]) -> Self:
         """实现反射加法操作。"""
 
         def generate() -> Iterator[Union[T, E]]:
@@ -160,10 +179,10 @@ class ArkoWrapper(Generic[T]):
             return True
         return self == [other]
 
-    def __copy__(self: Wrapper) -> Wrapper:
+    def __copy__(self) -> Self:
         return self.__class__(self.__root__)
 
-    def __deepcopy__(self: Wrapper, *args) -> Wrapper:
+    def __deepcopy__(self, *args) -> Self:
         """定义对类的实例使用 copy.copy() 时的行为"""
         return self.__class__(self._tee())
 
@@ -244,7 +263,7 @@ class ArkoWrapper(Generic[T]):
         """定义操作符(@)的行为。"""
         return self.__getitem__(other)
 
-    def __mul__(self: Wrapper, times: Union[int, float, str]) -> Wrapper:
+    def __mul__(self, times: Union[int, float, str]) -> Self:
         """实现乘法操作"""
         if isinstance(times, SupportsIndex):
             times = int(float(times))
@@ -257,11 +276,11 @@ class ArkoWrapper(Generic[T]):
         except Exception:
             raise TypeError(f"Unsupported Type: {type(times)}.")
 
-    def __neg__(self: Wrapper) -> Wrapper:
+    def __neg__(self) -> Self:
         """定义取负操作"""
         return self.__reversed__()
 
-    def __reversed__(self: Wrapper) -> Wrapper:
+    def __reversed__(self) -> Self:
         """定义反转"""
         if isinstance(self.__root__, Reversible):
             from copy import deepcopy as copy
@@ -306,14 +325,14 @@ class ArkoWrapper(Generic[T]):
         def accumulate(
                 self, func: Optional[Callable[[T, T], R]] = operator.add, *,
                 initial: Optional[int] = None
-        ) -> Wrapper:
+        ) -> Self:
             return self.__class__(
                 accumulate(self._tee(), func, initial=initial)
             )
     else:
         def accumulate(
                 self, func: Optional[Callable[[T, T], R]] = operator.add
-        ) -> Wrapper:
+        ) -> Self:
             return self.__class__(accumulate(self._tee(), func))
 
     def all(self) -> bool:
@@ -336,7 +355,7 @@ class ArkoWrapper(Generic[T]):
         except StopIteration:
             return False
 
-    def append(self, obj: E) -> Wrapper:
+    def append(self, obj: E) -> Self:
         """将对象附加到 ArkoWrapper 的末尾。注：此方法会修改本身的 __root__"""
 
         def clean() -> List[Union[E, T]]:
@@ -353,7 +372,7 @@ class ArkoWrapper(Generic[T]):
         self.__root__, result.__root__ = tee(clean())
         return self
 
-    def chain(self, *iterables: Iterable[E]) -> Wrapper:
+    def chain(self, *iterables: Iterable[E]) -> Self:
         """创建一个迭代器，它首先返回第一个可迭代对象中所有元素，接着返回下一个可迭代对象中所有元素，直到耗尽所有可迭代对象中的元素。"""
         return self.__class__(chain(self._tee(), *iterables))
 
@@ -365,28 +384,40 @@ class ArkoWrapper(Generic[T]):
         """以整个迭代器作为参数，于给定的 func 函数中进行运算"""
         return func(self._tee(), *args, **kwargs)
 
-    def combinations(self, r: int = 2) -> Wrapper:
+    def combinations(self, r: int = 2) -> Self:
         """返回由元素组成长度为 r 的子序列"""
         return self.__class__(combinations(self._tee(), r))
 
-    def combinations_with_replacement(self, r: int = 2) -> Wrapper:
+    def combinations_with_replacement(self, r: int = 2) -> Self:
         """返回由元素组成的长度为 r 的子序列，允许每个元素可重复出现。"""
         return self.__class__(combinations_with_replacement(self._tee(), r))
 
-    def compress(self, selectors: Iterable) -> Wrapper:
+    def compress(self, selectors: Iterable) -> Self:
         """返回元素中经 selectors 真值测试为 True 的元素"""
         return self.__class__(compress(self._tee(), selectors))
 
-    def cycle(self: Wrapper) -> Wrapper:
+    def cycle(self) -> Self:
         return self.__class__(cycle(self._tee()))
 
-    def drop_while(self: Wrapper, func: Callable[[T], bool]) -> Wrapper:
+    def delete(self, target: Union[Callable[[E], bool], E]) -> Self:
+        """从 ArkoWrapper 中删除指定对象。注：此方法会修改本身的 __root__"""
+
+        def generator() -> Iterator[E]:
+            is_callable = isinstance(target, Callable)
+            for e in iter(self.__root__):
+                if (is_callable and target(e)) or target == e:
+                    continue
+                yield e
+
+        return self.__class__(generator())
+
+    def drop_while(self, func: Callable[[T], bool]) -> Self:
         return self.__class__(dropwhile(func, self._tee()))
 
     def empty(self) -> bool:
         return self.__len__() == 0
 
-    def enumerate(self) -> Wrapper:
+    def enumerate(self) -> Self:
         def generator() -> Iterator[Tuple[int, T]]:
             iter_values = iter(self._tee())
             index = 0
@@ -400,7 +431,7 @@ class ArkoWrapper(Generic[T]):
 
         return self.__class__(generator())
 
-    def extend(self, iterable: Iterable[E]) -> Wrapper:
+    def extend(self, iterable: Iterable[E]) -> Self:
         if not isinstance(iterable, Iterable):
             raise TypeError(f"{type(iterable)} object is not iterable")
 
@@ -414,7 +445,7 @@ class ArkoWrapper(Generic[T]):
             self,
             num: int,
             factory: Union[R, Callable[[], R]] = None, *args, **kwargs
-    ) -> Wrapper:
+    ) -> Self:
         if num > self._max:
             raise ValueError("'num' cannot exceed the maximum number of '_max'")
         elif num < 0:
@@ -431,7 +462,7 @@ class ArkoWrapper(Generic[T]):
             self,
             num: int,
             factory: Union[R, Callable[[], R]] = None, *args, **kwargs
-    ) -> Wrapper:
+    ) -> Self:
         if num < self.length:
             raise ValueError("'num' cannot be less than its own length.")
         length = self.length
@@ -440,10 +471,10 @@ class ArkoWrapper(Generic[T]):
         else:
             return self.fill(num - length, factory=factory, *args, **kwargs)
 
-    def filter(self: Wrapper, func: Callable[[T], Any]) -> Wrapper:
+    def filter(self, func: Callable[[T], Any]) -> Self:
         return self.__class__(filter(func, self._tee()))
 
-    def filter_false(self: Wrapper, func: Callable[[T], Any]) -> Wrapper:
+    def filter_false(self, func: Callable[[T], Any]) -> Self:
         return self.__class__(filterfalse(func, self._tee()))
 
     def find(
@@ -462,23 +493,59 @@ class ArkoWrapper(Generic[T]):
                 if not full:
                     break
 
-    def flat(self, depth: int = -1) -> Wrapper:
-        def generator(iterator: Iterable, times: int) -> Iterator[T]:
-            for i in iterator:
-                if isinstance(i, Iterable) and not isinstance(i, str) and times:
-                    yield from generator(i, times - 1)
+    def flat(
+            self, depth: int = NOT_SET, *, flat_str: bool = False
+    ) -> "ArkoWrapper[Union[T, E]]":
+        def generator(
+                iterator: Iterable[E], times: int = NOT_SET
+        ) -> Iterator[Union[str, E, Iterator]]:
+            for item in iterator:
+                if isinstance(item, str) and times:
+                    if flat_str and len(item) != 0:
+                        yield from item
+                    else:
+                        yield item
+                elif isinstance(item, Iterable) and times:
+                    yield from generator(
+                        item, (times - 1) if isinstance(times, int) else times
+                    )
                 else:
-                    yield i
+                    yield item
 
         return self.__class__(generator(self._tee(), depth))
 
-    def group(self, n: int, fill_value: Any = None) -> Wrapper:
-        iter_values = [iter(self._tee())] * n
-        return self.__class__(zip_longest(*iter_values, fillvalue=fill_value))
+    def group(self, n: int, fill_value: Any = NOT_SET) -> "ArkoWrapper[Self]":
+        if not n:
+            raise ValueError(f"\'n\' must be a positive integer, not \'{n}\'")
+
+        def generator() -> Iterator[Self]:
+            iter_value = iter(self._tee())
+            stopped = False
+            next_value = NOT_SET
+            while not stopped:
+                item = self.__class__()
+                for _ in range(n):
+                    try:
+                        if next_value is not NOT_SET:
+                            item.append(next_value)
+                        else:
+                            item.append(next(iter_value))
+                    except StopIteration:
+                        stopped = True
+                        if fill_value is not NOT_SET:
+                            item.append(fill_value)
+                    try:
+                        next_value = next(iter_value)
+                    except StopIteration:
+                        stopped = True
+                        next_value = NOT_SET
+                yield item
+
+        return self.__class__(generator())
 
     # noinspection SpellCheckingInspection
     @overload
-    def groupby(self, key: Callable[[T], R] = None) -> Wrapper:
+    def groupby(self, key: Callable[[T], R] = None) -> "ArkoWrapper[Self]":
         pass
 
     # noinspection SpellCheckingInspection
@@ -488,12 +555,14 @@ class ArkoWrapper(Generic[T]):
             start: Callable[[T], bool],
             stop: Callable[[T], bool],
             group_type: Callable[[list], Iterable] = list,
-            retain: bool = False
-    ) -> Wrapper:
+            retain: bool = False,
+            contain_head: bool = False,
+            contain_tail: bool = False,
+    ) -> "ArkoWrapper[Self]":
         pass
 
     # noinspection SpellCheckingInspection
-    def groupby(self, *args, **kwargs) -> Wrapper:
+    def groupby(self, *args, **kwargs) -> "ArkoWrapper[Self]":
         if len(args) + len(kwargs) <= 1:
             return self.__class__(groupby(self._tee(), *args, **kwargs))
         else:
@@ -504,7 +573,7 @@ class ArkoWrapper(Generic[T]):
                     retain: bool = False,
                     contain_head: bool = False,
                     contain_tail: bool = False
-            ) -> Wrapper:
+            ) -> Self:
                 def generator() -> Iterator[Iterable[T]]:
                     elem = []
                     other = []
@@ -531,20 +600,22 @@ class ArkoWrapper(Generic[T]):
 
             return make_group(*args, **kwargs)
 
-    def join(self, sep: str = ', ') -> str:
+    def join(self, sep: E = ', ') -> "ArkoWrapper[Union[T, E]]":
+        """在每个元素之间加上 sep """
         iter_values = iter(self._tee())
-        result = ''
-        try:
+
+        def generator() -> Iterator[Union[T, E]]:
             for _ in range(self.max_operate_time):
-                value = next(iter_values)
-                result += f"{sep}{value}"
-        except StopIteration:
-            ...
-        return result
+                try:
+                    yield next(iter_values)
+                except StopIteration:
+                    break
+                yield sep
+        return self.__class__(generator())
 
     def map(
             self, func: Callable[[T], R], start: Optional[int] = 0
-    ) -> Wrapper:
+    ) -> Self:
         def generator() -> Iterator[T]:
             iter_values = iter(self._tee())
             for _ in range(start):
@@ -556,17 +627,17 @@ class ArkoWrapper(Generic[T]):
     def mutate(
             self, func: Callable[[Iterable[T], Any], Iterable] = list,
             *args, **kwargs
-    ) -> Wrapper:
+    ) -> Self:
         """改变 __root__ 的类型。"""
         self.__root__ = func(self.__root__, *args, **kwargs)
         return self
 
     def print(
-            self: Wrapper,
+            self,
             length: Optional[int] = None, *,
             end: Optional[str] = ', ',
-            print_func: Optional[Callable[..., Any]] = print
-    ) -> Wrapper:
+            print_func: Optional[Callable[..., Any]] = builtins.print
+    ) -> Self:
         time = 0
         iter_values = iter(self._tee())
         try:
@@ -585,7 +656,7 @@ class ArkoWrapper(Generic[T]):
             print_func(len(end) * '\b')
         return self
 
-    def remove(self: Wrapper, target: Any, *, full: bool = False) -> Wrapper:
+    def remove(self, target: Any, *, full: bool = False) -> Self:
         def generator() -> Iterator[T]:
             iter_values = iter(self._tee())
             is_sequence = isinstance(target, Sequence)
@@ -611,8 +682,8 @@ class ArkoWrapper(Generic[T]):
         return self.__class__(generator())
 
     def repeat(
-            self: Wrapper, times: Optional[Union[int, float, str]] = None
-    ) -> Wrapper:
+            self, times: Optional[Union[int, float, str]] = None
+    ) -> Self:
         if times is None:
             return self.__class__(
                 chain.from_iterable(
@@ -632,20 +703,20 @@ class ArkoWrapper(Generic[T]):
         else:
             raise TypeError(f"Unsupported Type: {type(times)}.")
 
-    def reverse(self: Wrapper) -> Wrapper:
+    def reverse(self) -> Self:
         return self.__reversed__()
 
     @overload
-    def slice(self: Wrapper, stop: int) -> Wrapper:  # pragma: no cover
+    def slice(self, stop: int) -> Self:  # pragma: no cover
         ...
 
     @overload
     def slice(
-            self: Wrapper, start: int, stop: int, step: Optional[int] = 1
-    ) -> Wrapper:  # pragma: no cover
+            self, start: int, stop: int, step: Optional[int] = 1
+    ) -> Self:  # pragma: no cover
         ...
 
-    def slice(self: Wrapper, *args, **kwargs) -> Wrapper:
+    def slice(self, *args, **kwargs) -> Self:
         return self.__class__(islice(self._tee(), *args, *kwargs.values()))
 
     def _iter_integer(self, start: int = 0) -> Iterator[int]:
@@ -674,6 +745,7 @@ class ArkoWrapper(Generic[T]):
 
         j = 0
 
+        # noinspection PyProtectedMember
         for i in target._iter_integer():
             while j > 0 and target[i] != sub[j]:
                 j = partial[j - 1]
@@ -684,16 +756,16 @@ class ArkoWrapper(Generic[T]):
                 j = partial[j - 1]
 
     def sort(
-            self: Wrapper, key: Optional[Callable] = None, reverse: bool = False
-    ) -> Wrapper:
+            self, key: Optional[Callable] = None, reverse: bool = False
+    ) -> Self:
         return self.__class__(sorted(self._tee(), key=key, reverse=reverse))
 
-    def starmap(self, func: Callable[[T, T], R]) -> Wrapper:
+    def starmap(self, func: Callable[[T, T], R]) -> Self:
         return self.__class__(starmap(func, self._tee()))
 
     def take_while(
-            self: Wrapper, func: Union[Any, Callable[[T], bool]] = True
-    ) -> Wrapper:
+            self, func: Union[Any, Callable[[T], bool]] = True
+    ) -> Self:
         if callable(func):
             return self.__class__(takewhile(func, self._tee()))
         elif bool(func):
@@ -702,13 +774,13 @@ class ArkoWrapper(Generic[T]):
             return self.__class__()
 
     def tee(
-            self: Wrapper, n: Optional[int] = None
-    ) -> Union[Wrapper, Iterable[Wrapper]]:
+            self, n: Optional[int] = None
+    ) -> Union[Self, Iterable[Self]]:
         if n is None:
             return self.__class__(self._tee())
         return (self.__class__(item) for item in tee(self.__root__, n))
 
-    def unique(self: Wrapper, key: Optional[Callable] = None) -> Wrapper:
+    def unique(self, key: Optional[Callable] = None) -> Self:
         def generator() -> Iterator[T]:
             for k, g in groupby(sorted(self._tee(), key=key)):
                 yield k
@@ -732,16 +804,16 @@ class ArkoWrapper(Generic[T]):
         def zip(
                 self, *iterables: Iterable[E],
                 strict: Optional[bool] = False
-        ) -> Wrapper:
+        ) -> Self:
             return self.__class__(zip(self._tee(), *iterables, strict=strict))
     else:
-        def zip(self, *iterables: Iterable[E]) -> Wrapper:
+        def zip(self, *iterables: Iterable[E]) -> Self:
             return self.__class__(zip(self._tee(), *iterables))
 
     def zip_longest(
             self, *iterables: Iterable[E],
             fill_value: Optional[R] = None
-    ) -> Wrapper:
+    ) -> Self:
         return self.__class__(
             zip_longest(
                 self.tee(), *ArkoWrapper(iterables).tee(), fillvalue=fill_value
